/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/


using System;
using System.Reflection;

using System.Collections.Generic;

using System.Text;

using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;


namespace EvoLab.Common
{

    [XmlRoot("dictionary")]
    [Serializable]
    public class SerializableDictionary<TKey, TValue>

    : Dictionary<TKey, TValue>, IXmlSerializable, ISerializable, IDeserializationCallback
    {

        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }



        public void ReadXml(System.Xml.XmlReader reader)
        {
            XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            // WB: 20.08.2007 dieser Aufruf wirft eine Exception !
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

            bool wasEmpty = reader.IsEmptyElement;
            reader.Read();

            if (wasEmpty)
                return;

            while (reader.NodeType != System.Xml.XmlNodeType.EndElement)
            {
                reader.ReadStartElement("item");
                reader.ReadStartElement("key");
                TKey key = (TKey)keySerializer.Deserialize(reader);
                reader.ReadEndElement();
                reader.ReadStartElement("value");
                TValue value = (TValue)valueSerializer.Deserialize(reader);
                reader.ReadEndElement();
                this.Add(key, value);
                reader.ReadEndElement();
                reader.MoveToContent();
            }

            reader.ReadEndElement();

        }



        public void WriteXml(System.Xml.XmlWriter writer)
        {
            XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
            XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

            foreach (TKey key in this.Keys)
            {

                writer.WriteStartElement("item");

                writer.WriteStartElement("key");
                keySerializer.Serialize(writer, key);
                writer.WriteEndElement();

                writer.WriteStartElement("value");
                TValue value = this[key];
                valueSerializer.Serialize(writer, value);
                writer.WriteEndElement();

                writer.WriteEndElement();

            }
        }
        
        public void WriteCSV(EvoLab.Common.CSVWriter writer)
        {
            TValue value = default(TValue);
            
            foreach (TKey key in this.Keys)
            {
                    value = this[key];
                    writer.WriteCells(key.ToString(), value.ToString());
            }
        }
        
        #endregion

        public SerializableDictionary() : base() { }

        public SerializableDictionary(IDictionary < TKey, TValue > dictionary) : base(dictionary) { }

        public SerializableDictionary(IEqualityComparer < TKey > comparer) : base(comparer) { }

        public SerializableDictionary(int capacity) : base(capacity) {}

        public SerializableDictionary(IDictionary<TKey, TValue> dictionary, IEqualityComparer<TKey> comparer) : base(dictionary, comparer) { }

        public SerializableDictionary(int capacity,IEqualityComparer < TKey > comparer) : base(capacity, comparer) {}
        
        protected SerializableDictionary(SerializationInfo si, StreamingContext context)
            : base(si, context)
        {}

        void ISerializable.GetObjectData (SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData (info, context);
        }

    }
}
