/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

namespace EvoLab.CoreApi {

	public enum ModuleType {
		DEFAULT,
		RECOMBINATION,
		SELECTION
	}
	
	public enum PopSizeType {
		Any,
		Specific
	}
	
	public sealed class EAModuleInfo {
		public readonly ModuleType type;
		public readonly PopSizeType sourcePopSizeType;
		public readonly int sourcePopSize;
		
		public static readonly EAModuleInfo DefaultInfo = new EAModuleInfo (ModuleType.DEFAULT, PopSizeType.Any, 0);
		
		private EAModuleInfo (ModuleType type, PopSizeType sourcePopSizeType, int sourcePopSize) {
			this.type = type;
			this.sourcePopSize = sourcePopSize;
			this.sourcePopSizeType = sourcePopSizeType;
		}
		
		public static EAModuleInfo CreateSelectionInfo () {
			return new EAModuleInfo (ModuleType.SELECTION, PopSizeType.Any, 0);
		}
		
		public static EAModuleInfo CreateRecombinationInfo () {
			return new EAModuleInfo (ModuleType.RECOMBINATION, PopSizeType.Any, 0);
		}
		
		public static EAModuleInfo CreateRecombinationInfo (int numberOfParentsNeeded) {
			return new EAModuleInfo (ModuleType.RECOMBINATION, PopSizeType.Specific, numberOfParentsNeeded);
		}
	}
}
