/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System.IO;

namespace EvoLab.CoreApi {

    /// <summary>
    /// Dieses Interface erleichtert den konfliktfreien Zugriff auf
    /// hochgeladene Ressourcendateien.
    /// </summary> 
    /// @cond
    /// <author>Daniel Hofmann</author>
    /// <version>$Revision: 2634 $</version>
    /// <id>$Id: IResourceAccess.cs 2634 2008-04-28 15:28:13Z hofmandl $</id>
    /// @endcond
    public interface IResourceAccess {

        /// <summary>
        /// Öffnet den angegebenen Dateinamen im jeweiligen
        /// Ressourcenverzeichnis.
        /// </summary>
        /// <param name="fileName">Der Name der Datei ohne den Pfad.</param>
        /// <returns>Read-Only-FileStream mit dem Dateiinhalt.</returns>
        FileStream OpenRead (string fileName);

        /// <summary>
        /// Öffnet die angegebene Textdatei im jeweiligen
        /// Ressourcenverzeichnis.
        /// </summary>
        /// <param name="fileName">Der Name der Datei ohne den Pfad.</param>
        /// <returns>StreamReader mit dem Dateitext als Inhalt.</returns>
        StreamReader OpenText (string fileName);

        /// <summary>
        /// Das persönliche Ressourcenverzeichnis des jeweiligen
        /// Benutzers.
        /// Sämtliche hochgeladenen Dateien (mit Ausnahme der Module)
        /// landen in diesem Verzeichnis. Für Simulationen sollte man
        /// sich auf die Verwendung dieses Verzeichnisses beschränken.
        /// </summary>
        string ResourceFolder {
            get;
        }
    }

}
