/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace EvoLab.CoreApi.ModuleTemplates {

    public abstract class EAFitnessBase<I, P>
        : IModuleFitness<P>
        where P : EAPopulation<I>
        where I : ITransformable<I>
    {
        protected IWorld world;
        protected string name;
        protected ISimulationControl simManagement;
		protected P population;

        public virtual void Init(
            string name, P population,
            IWorld world,
            ISimulationControl simManagement)
        {
			if (name == null)
				throw new ArgumentNullException("name");
			if (population == null)
				throw new ArgumentNullException("population");
			if (world == null)
				throw new ArgumentNullException("world");
			if (simManagement == null)
				throw new ArgumentNullException("simManagement");
			
            this.name = name;
            this.world = world;
            this.simManagement = simManagement;
			this.population = population;
			
			population.RegisterModule(name, EAModuleInfo.DefaultInfo);
        }

		public virtual void Dispose () {}
		
        public abstract void Run ();
    }
}
