/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleInterfaces;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Populations;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.ModuleTemplates {

    [DoubleParameter("MUTATION_PROBABILITY", 0.8d)]
    public abstract class EAMutation<I, P>
        : EAMutationBase<I, P>
        where P : EAPopulation<I>
        where I : ITransformable<I>
    {
        protected abstract void Mutate (I mutant);

        private double mutationProbability;
        
        public override void Init(
            string name, P population,
            IWorld world)
        {
			base.Init(name, population, world);
			
            mutationProbability = world.GetLocalDouble(name, "MUTATION_PROBABILITY");

            if (mutationProbability < 0.0d || mutationProbability > 1.0d)
                throw new ArgumentOutOfRangeException (
                    "Parameter MUTATION_PROBABILITY liegt nicht zwischen 0.0 und 1.0");
        }

        public override void Run ()
        {
			foreach (I ind in population.Pop) {
				if (world.RandomNumberGenerator.NextDouble() <= mutationProbability) {
					Mutate (ind);
				}
			}
        }
    }
}
