/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.Random;
using EvoLab.CoreApi.Individuals;

namespace EvoLab.CoreApi.Operators
{
    public class Initialization {

        public static void RandomizeFloatArray
        (IRandom rand, float[] array)
        {
            if (array == null)
                throw new ArgumentNullException ("array");
            if (rand == null)
                throw new ArgumentNullException ("rand");

            for (int i=0; i < array.Length; ++i) {
                array[i] = (float)(rand.NextDouble() + rand.Next(Int32.MinValue, Int32.MaxValue));
            }
        }

        public static void RandomizeDoubleArray
        (IRandom rand, double[] array)
        {
            RandomizeDoubleArray(rand, array, (double)Int32.MinValue, (double)Int32.MaxValue);
        }

        public static void RandomizeDoubleArray
        (IRandom rand, double[] array, double min, double max)
        {
            if (array == null)
                throw new ArgumentNullException ("array");
            if (rand == null)
                throw new ArgumentNullException ("rand");

            for (int i=0; i < array.Length; ++i) {
                array[i] = rand.NextDouble() * (max - min) + min;
            }
        }

        public static void RandomizeIntPermutation
        (IRandom rand, int[] array)
        //TODO: Use InversionVectors
        {
            if (array == null)
                throw new ArgumentNullException ("array");
            if (rand == null)
                throw new ArgumentNullException ("rand");

            int arrayLength = array.Length;

            //TODO: Use stackbits
            bool[] usedValues = new bool[arrayLength];
            for (int k=0; k<arrayLength; k++) {
                usedValues[k] = false;
            }

            for (int j=0; j < arrayLength; j++) {
                int value = rand.Next(arrayLength);
                while (usedValues[value])
                {
                    value = rand.Next(arrayLength);
                }
                usedValues[value] = true;

                array[j] = value;
            } 
        }
    }
}
