/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using System.Collections.Generic;

using EvoLab.Common;
using EvoLab.CoreApi.Parameters;

namespace EvoLab.CoreApi.Populations
{

    [Serializable]
    public class SimRepresentative : ICloneable<SimRepresentative> {

        public bool mayBeParent = false;
        public bool needsEvaluation = true;
        public bool isChild = false;
        public double fitness;

        public bool simulationFailed = false;
        public SimConfig simConfig;

        public SimRepresentative Clone ()
        {
            SimRepresentative clone = new SimRepresentative ();
            clone.mayBeParent = mayBeParent;
            clone.isChild = isChild;
            clone.needsEvaluation = needsEvaluation;
            clone.fitness = fitness;
            clone.simulationFailed = simulationFailed;
            clone.simConfig = simConfig.Clone ();
            return clone;
        }
    }

    [Description (@"Die SimulationPopulation ist darauf ausgelegt, mehr Freiraum beim Modulprogrammieren und Modulauswählen zu geben,
da die Performanznachteile, die hierdurch entstehen gegenüber der aufwändigen Fitnessberechnung kaum ins Gewicht fallen,
solange die Gesamtzahl der Individuen relativ klein ist.
Es gibt eine einzige Liste von Individuen. Sämtliche Zustandsinformation stecken im verwendeten Individuumstyp selbst.

Dadurch sind die Module praktisch in beliebigen Reihenfolgen anordenbar, auch wenn nicht jede Sinn macht.

Modulkonventionen:
Das Initialisierungsmodul erstellt eine beliebige Anzahl von Individuen in pop. Dabei darf nur das SimConfig-Feld der einzelnen Individuen
verändert werden.

Elternselektion setzt oder entfernt bei einer beliebigen Teilmenge der Individuen, die bereits evaluiert sind das Eltern-Flag.

Rekombination fügt zu pop eine beliebige Menge neuer Individuen hinzu, wobei für neue Individuen die Standardwerte gesetzt sein müssen,
außer dem Kind-flag, welches gesetzt sein muss, und der SimConfig, die sich aus denen der Eltern zusammensetzen muss.
Nach Ende einer Rekombination müssen alle mayBeParent-Flags auf false stehen.

Mutation darf beliebig viele Individuen direkt mutieren, oder eine Kopie des Individuums machen und erst darin die SimConfig verändern.
Im Mutant muss anschließend das Evaluierungsbit auf true stehen.

Die Fitnessbewertung muss alle Individuen bewerten, deren Evaluierungsbit auf true steht.
Das Evaluierungsbit muss anschließend auf false gesetzt sein und das simulationFailed-Bit auf true gdw. das Simulieren mit der SimConfig zu einem
Fehler führte. Für die Fitnesswerte gilt: Ein höherer Wert ist fitter, wobei bei fehlgeschlagener Simulation keiner gesetzt werden muss.

Umweltselektion darf unter Zuhilfenahme aller Werte aller Individuen eine beliebige Menge von Individuen aus der Liste löschen.
Am Ende müssen alle Kind-flags und alle MayBeParent-Flags auf false stehen.
")]

    [Serializable]
    public class SimulationPopulation {

        public readonly List<SimRepresentative> pop = new List<SimRepresentative> ();
        
    }

}
