/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Problems;

namespace EvoLab.CoreApi.Populations {

    [Identifier ("RucksackChargePopulation")]

    [Serializable]
    public class RucksackChargePopulation : EAPopulation <RucksackCharge> {

        public readonly RucksackProblem problem;

        public RucksackChargePopulation (RucksackProblem problem, int μ, int λ)
            : base (μ, λ, (new RucksackChargeFactory(problem.objects.Length)))
        {
            this.problem = problem;
        }
    }

    internal class RucksackChargeFactory : IndividualFactory<RucksackCharge>
    {
        private readonly int length;

        internal RucksackChargeFactory (int length)
        {
            this.length = length;
        }

        public RucksackCharge allocateNewIndividual ()
        {
            RucksackCharge rc = new RucksackCharge ();
            rc.charge = new bool [length];
            return rc;
        }
    }
}
