/******************************************************************************
 * Copyright (c) 2008 Christian Fonden, Tim Schumacher, Tobias Fink,          *
 *                    Ting Wang, Daniel Hofmann, Vasilios Grigas,             *
 *                    Michael Fechter, Wolfgang Bischoff                      *
 *                                                                            *
 *  This file is part of EvoLab. (Library)                                    *
 *                                                                            *
 *  EvoLab-Library is free software: you can redistribute it and/or modify    *
 *  it under the terms of the MIT/X11 license as published by                 *
 *  the Massachusetts Institute of Technology.                                *
 *                                                                            *
 *  EvoLab is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  MIT/X11 License for more details.                                         *
 *                                                                            *
 *  You should have received a copy of the MIT/X11 License along with EvoLab. *
 *  If not, see <http://www.opensource.org/licenses/mit-license.php>.         *
 ******************************************************************************/

using System;

using EvoLab.CoreApi.ModuleTemplates;
using EvoLab.CoreApi.Parameters;
using EvoLab.CoreApi.Individuals;
using EvoLab.CoreApi.Populations;

namespace EvoLab.Core.StandardModules.Universal
{
	[Identifier("EA_Permutation_MutationInversion")]
	
    public class PermutationMutationInversion<I, P> : EAMutation<I, P> where P : VectorPopulation<I, int> where I : Vector<I, int>
    {
        protected override void Mutate (I mutant)
        {
            int [] perm = mutant.vector;
            int permutationLength = perm.Length;
            int startIndex = world.RandomNumberGenerator.Next(permutationLength);
            int endIndex = world.RandomNumberGenerator.Next(permutationLength);

            if (startIndex == endIndex)
                return;

            int meetingIndex;
            if (startIndex < endIndex) {
                meetingIndex = (startIndex + endIndex) / 2;
            } else {
                meetingIndex = startIndex + ((permutationLength - startIndex + endIndex) / 2);
                if (meetingIndex >= permutationLength)
                    meetingIndex -= permutationLength;
            }

            int i = startIndex;
            int j = endIndex;
            while (j != meetingIndex) {

                int tempValue = perm[i];
                perm[i] = perm[j];
                perm[j] = tempValue;

                ++i; --j;

                if (i == permutationLength)
                    i = 0;
                if (j == -1)
                    j = permutationLength - 1;
            }
        }
    }
}
